import tornado.web
import sqlite3
import json
import os

# TODO 并发性能问题
# sqlite3.connect()
# 当一个数据库被多个连接访问，且其中一个修改了数据库，此时 SQLite 数据库被锁定，直到事务提交。
# timeout 参数表示连接等待锁定的持续时间，直到发生异常断开连接。timeout 参数默认是 5.0（5 秒）。

DATABASE = ''
DATABASE_DQL_BASEURI = 'dql'
DATABASE_DCL_BASEURI = 'dcl'


def _ddl(sql):
    return

# dql不需要commit
def _dql(sql):
    print("_dql", sql)
    conn = sqlite3.connect(DATABASE)
    c = conn.cursor()
    cursor = c.execute(sql)
    result = list(cursor)
    # for row in cursor:
    #     print "ID = ", row[0]
    #     print "NAME = ", row[1]
    #     print "ADDRESS = ", row[2]
    #     print "SALARY = ", row[3], "\n"
    conn.close()
    print(result)
    return result
    

class DQLHandler(tornado.web.RequestHandler):
    def initialize(self, db):
        print(db)
        self.db = db

    def post(self):
        self.set_header("Content-Type", "text/plain")
        # curl -X POST http://localhost:8888/dql -d '{"sql":"SELECT * FROM BLOG"}'
        param = self.request.body.decode('utf-8')
        param = json.loads(param)
        print(param)
        result = _dql(param['sql'])
        self.write(json.dumps(result, ensure_ascii=False))


def _dcl(sql):
    print("_dcl", sql)
    conn = sqlite3.connect(DATABASE)
    c = conn.cursor()
    print ("数据库打开成功")
    cursor = c.execute(sql)
    print(cursor.fetchall())
    result = list(cursor)
    conn.commit()
    print ("数据插入成功")
    conn.close()
    print(result)
    return True

class DCLHandler(tornado.web.RequestHandler):
    def initialize(self, db):
        print(db)
        self.db = db

    def post(self):
        self.set_header("Content-Type", "text/plain")
        # curl -X POST http://localhost:8888/bloggart/dcl -d '{"sql":"INSERT INTO BLOG (DATETIME,NAME,MEMO,HEADIMG,LINK,CLASS) VALUES (\'7\', \'8\', \'9\', \'10\', \'11\', \'12\');"}'
        param = self.request.body.decode('utf-8')
        param = json.loads(param)
        print(param)
        result = _dcl(param['sql'])
        self.write(json.dumps(result, ensure_ascii=False))

def init(sqlpath = ""):
    print("loading sql file from:", sqlpath)
    conn = sqlite3.connect(DATABASE)
    conn.executescript(open(sqlpath,'r').read())
    conn.commit()
    print ("数据插入成功")
    conn.close()
    return

def Handle(config):
    global DATABASE
    DATABASE = config['path']

    if 'init' in config:
        init(config['init'])

    urls = [
        # 我们修改了原先的映射规则，将默认页请求映射到test的规则修改成，访问相对目录/test
        (os.path.join(r"/", config['base_uri'], DATABASE_DQL_BASEURI), DQLHandler, dict(db=DATABASE)),
        (os.path.join(r"/", config['base_uri'], DATABASE_DCL_BASEURI), DCLHandler, dict(db=DATABASE)),
    ]
    return urls



import os
import json
import tornado
# from bloggart.core import config

class DQLHandler(tornado.web.RequestHandler):
    def initialize(self, db):
        print(db)
        self.db = db

    # def get(self, table, offset, limit):
    #     _dql(table, offset, limit)

    def post(self):
        self.set_header("Content-Type", "text/plain")
        # curl -X POST http://localhost:8888/dql -d 'message=d'
        # self.write("You wrote " + self.get_body_argument("message"))

        # curl -X POST http://localhost:8888/dql -d '{"message":"d"}'
        param = self.request.body.decode('utf-8')
        param = json.loads(param)
        print(param)
        _dql(param['sql'])
        self.write('ok')
 
# import tornado.web
# import os

# FILESERVER_BASEURI = 'static'

# class MainHandler(tornado.web.RequestHandler):
#     def get(self):
#         self.write("Hello, world")

# def Init(config):
#     uri = os.path.join(r"/", config['base_uri'], FILESERVER_BASEURI)
#     print(config, uri)
#     return [(uri, MainHandler)]


# -*- coding: UTF-8 -*-
import configparser
import os
import re

# 文件编码
FILE_ENCODE = 'UTF-8'
CONFIG = configparser.ConfigParser()

def Parse(path):
    global CONFIG

    # 这些相当于是参数的默认值
    return CONFIG.read(path, encoding=FILE_ENCODE)
    
def Get_fileserver():
    if 'FILESERVER' in CONFIG:
        return dict(CONFIG['FILESERVER'])
    else:
        raise Exception('FILESERVER not in CONFIG!')


def Get_database():
    if 'DATABASE' in CONFIG:
        return dict(CONFIG['DATABASE'])
    else:
        raise Exception('DATABASE not in CONFIG!')
        
        
    # DICTIONARY = set()
    # if 'DICTIONARY' in CONFIG['DEFAULT']:
    #     DICTIONARY = set(readlist(os.path.join(os.path.dirname(__file__), CONFIG['DEFAULT']['DICTIONARY'])))


    # if 'DATABASE' in CONFIG['DEFAULT']:
    #     MAX_ERROR_TIMES_PERTAG_PERTYPE_CSV = int(CONFIG['DEFAULT']['MAX_ERROR_TIMES_PERTAG_PERTYPE_CSV'])

    # if 'PROXY' in CONFIG['DEFAULT']:
    #     MAX_LENGTH_PERTAG = int(CONFIG['DEFAULT']['MAX_LENGTH_PERTAG'])

    # if 'REALTIME' in CONFIG['DEFAULT']:
    #     MAX_LENGTH_PERTAG = int(CONFIG['DEFAULT']['MAX_LENGTH_PERTAG'])

    # return CONFIG['DEFAULT'], DICTIONARY, PATTERNS, MAX_ERROR_TIMES_PERTAG_PERTYPE, MAX_ERROR_TIMES_PERTAG_PERTYPE_CSV, MAX_LENGTH_PERTAG


    # PATTERNS = []
    # for field in CONFIG:
    #     if field == 'DEFAULT':
    #         continue
    #     if 'mode' not in CONFIG[field]:
    #         continue
    #     t = {}
    #     t['name'] = field
    #     t['mode'] = CONFIG[field]['mode']
    #     t['stat'] = CONFIG[field]['stat']
    #     t['patterns'] = []
    #     t['escapes']  = []
    #     t['escape_pfx'] = []
    #     t['escape_sfx'] = []
    #     for item in CONFIG[field]:
    #         if item.startswith('__pattern__'):
    #             if os.path.isfile(os.path.join(os.path.dirname(__file__), CONFIG[field][item])):
    #                 dictionary = readlist(os.path.join(os.path.dirname(__file__), CONFIG[field][item]))
    #                 dictionary = [trans_reserve(i) for i in dictionary]
    #                 t['patterns'].append(re.compile(" |".join(dictionary)))
    #             else:
    #                 t['patterns'].append(re.compile(CONFIG[field][item]))
    #         if item.startswith('__escape__'):
    #             t['escapes'].append(re.compile(CONFIG[field][item]))
    #         if item.startswith('__escape_pfx__'):
    #             t['escape_pfx'].append(re.compile(CONFIG[field][item]))
    #         if item.startswith('__escape_sfx__'):
    #             t['escape_sfx'].append(re.compile(CONFIG[field][item]))
    #     PATTERNS.append(t)



"""bloggart command line interface."""
from bloggart import header

# def main():
#     app=make_app()  #建立Application对象
#     app.listen(8888) #设置监听端口
#     try:
#         #启动IOLoop
#         tornado.ioloop.IOLoop.current().start()
#     except KeyboardInterrupt:
#         tornado.ioloop.IOLoop.current().stop()
#         #此处执行资源回收工作
#         print("Program exit!")

def main():
    """Entry point for the application script"""
    print("Call your main application code here")
    header.Run()
    exit()

    # args = parser.parse_args()

    # if args.quiet:
    #     jieba.setLogLevel(60)
    # if args.pos:
    #     import jieba.posseg
    #     posdelim = args.pos
    #     def cutfunc(sentence, _, HMM=True):
    #         for w, f in jieba.posseg.cut(sentence, HMM):
    #             yield w + posdelim + f
    # else:
    #     cutfunc = jieba.cut

    # delim = text_type(args.delimiter)
    # cutall = args.cutall
    # hmm = args.hmm
    # fp = open(args.filename, 'r') if args.filename else sys.stdin

    # if args.dict:
    #     jieba.initialize(args.dict)
    # else:
    #     jieba.initialize()
    # if args.user_dict:
    #     jieba.load_userdict(args.user_dict)

    # ln = fp.readline()
    # while ln:
    #     l = ln.rstrip('\r\n')
    #     result = delim.join(cutfunc(ln.rstrip('\r\n'), cutall, hmm))
    #     if PY2:
    #         result = result.encode(default_encoding)
    #     print(result)
    #     ln = fp.readline()

    # fp.close()




import tornado.ioloop
import tornado.web

def Serving(port, handlers):
    for h in handlers:
        print(h[0])
    application = tornado.web.Application(handlers)
    application.listen(port)
    tornado.ioloop.IOLoop.current().start()


import tornado.web
import os

FILESERVER_BASEURI = 'html'

# class MainHandler(tornado.web.RequestHandler):
#     def get(self):
#         self.write("Hello, world")

def Handle(config):
    if not os.path.isdir(config['static']):
        raise Exception('static is not directory: {}'.format(config['static']))
    # utils.simplifyPath(config['static'])
    uri = os.path.join(r"/", config['base_uri'], FILESERVER_BASEURI, r'(.*)$')
    print(config, uri)
    # return [(r'/(.*)$', tornado.web.StaticFileHandler, {'path': 'html/', 'default_filename': 'index.html'})]
    return [(uri, tornado.web.StaticFileHandler, {'path': config['static'], 'default_filename': 'index.html'})]


def print_hello(port):
    print('==> Deploying bloggart:')
    print('http://localhost:' + str(port) + "/")


def readlist(path):
    if (path == ''):
        return []
    lines = [line.strip() for line in open(path, 'r', encoding=FILE_ENCODE).readlines()]
    return lines

def trans_reserve(line):
    reserved_chars = '''?&|!{}[]()^~*:\\"'+-.+'''
    replace = ['\\' + l for l in reserved_chars]
    trans = dict(zip(reserved_chars, replace))
    result = ''
    for char in line:
        if char in trans:
            result += trans[char]
        else:
            result += char
    return result

# 71. 简化路径
# 作者：LeetCode-Solution
# 链接：https://leetcode-cn.com/problems/simplify-path/solution/jian-hua-lu-jing-by-leetcode-solution-aucq/
# 来源：力扣（LeetCode）
def simplifyPath(path: str) -> str:
    names = path.split("/")
    stack = list()
    for name in names:
        if name == "..":
            if stack:
                stack.pop()
        elif name and name != ".":
            stack.append(name)
    return "/".join(stack)





import tornado.ioloop
import tornado.web
from bloggart.core import config
from bloggart.core import database
from bloggart.core import fileserver
from bloggart.core import service
# import socket
# g_server_ip = socket.gethostbyname(socket.gethostname())
import argparse

def get_args_parser():
    parser = argparse.ArgumentParser(description="bloggart command line interface.")
    parser.add_argument("-c", "--config", default='config.ini', help="配置文件路径")
    return parser.parse_args()

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.write("Hello, world")

def Run():
    args = get_args_parser()
    config.Parse(args.config)

    handler = []
    handler.extend(database.Handle(config.Get_database()))
    handler.extend(fileserver.Handle(config.Get_fileserver()))
    service.Serving(handler)
    
    print(config.CONFIG)
    exit()
    
if __name__ == "__main__":
    Run()
